<?php
/*
  stats_low_stock_attrib.php

  QTPro 7.4.0 CE Phoenix
  by @raiwa
  info@oscaddons.com
  www.oscaddons.com

  Copyright (c) 2021 Rainer Schmied

  Contribution extension to:
  CE Phoenix, E-Commerce made Easy
  https://phoenixcart.org

  Copyright (c) 2021 Phoenix Cart

  Copyright (c) 2004 Ralph Day
  Released under the GNU General Public License

  Based on prior works released under the GNU General Public License:
    QT Pro prior versions
      Ralph Day, October 2004
      Tom Wojcik aka TomThumb 2004/07/03 based on work by Michael Coffman aka coffman
      FREEZEHELL - 08/11/2003 freezehell@hotmail.com Copyright (c) 2003 IBWO
      Joseph Shain, January 2003
    osCommerce MS2
      Copyright (c) 2003 osCommerce

  Modifications made:
    11/2004 - Clean up to not replicate for all languages
              Handle multiple attributes per product
              Ignore attributes that stock isn't tracked for
              Remove unused code

*******************************************************************************************
  QT Pro Low Stock Report

  This report lists all products and products attributes that have stock less than
  the reorder level configured in the osCommerce admin site
*******************************************************************************************

  $Id: stats_products.php,v 1.22 2002/03/07 20:30:00 harley_vb Exp $
  (v 1.3 by Tom Wojcik aka TomThumb 2004/07/03)
  CE Phoenix, E-Commerce made Easy
  https://phoenixcart.org

  Copyright (c) 2021 Phoenix Cart

  Released under the GNU General Public License
*/
  require('includes/application_top.php');

	$product_investigation = (isset($_GET['pID']))? qtpro_doctor_investigate_product($_GET['pID']) : null;
	$qtpro_sick_count = qtpro_sick_product_count();
	if ($qtpro_sick_count != 0) {
	  $messageStack->add(sprintf(HOOK_SWQTPRO_WARNING, $qtpro_sick_count, $Admin->link('qtprodoctor.php')), 'error');
  }

  require('includes/template_top.php');

  echo '<style>.table-not-striped tbody tr:nth-of-type(odd){background-color: transparent!important;}.table-not-striped, .table-not-striped tbody tr {line-height: 1;!important;}</style>';

  if ( !defined('MODULE_CONTENT_PI_QTPRO_OPTIONS_STATUS') && !defined('PI_QTPRO_OPTIONS_STATUS') ) {
    echo '<div class="secWarning">' . QTPRO_OPTIONS_WARNING . '<br>
          <a href="modules_content.php?module=cm_pi_qtpro_options&action=install">' . QTPRO_OPTIONS_INSTALL_NOW . '</a></div>';
  }
  if (!file_exists(DIR_FS_CATALOG . 'templates/' . TEMPLATE_SELECTION . '/includes/hooks/shop/siteWide/qtPro.php')) {
    echo '<div class="secWarning"><br>' . QTPRO_ERROR_SHOP_HOOK . '<br></div>';
  }
  if (!file_exists(DIR_FS_CATALOG . 'includes/hooks/admin/siteWide/swQtPro.php')) {
    echo '<div class="secWarning"><br>' . QTPRO_ERROR_ADMIN_HOOK . '<br></div>';
  }

  $currencies = new currencies();
?>
  <h1 class="display-4 mb-2"><?= HEADING_TITLE ?></h1>

  <div class="d-flex justify-content-center mt-3">
  <div class="table-responsive col-lg-10 col-lg-offset-1">
    <table class="table table-sm table-bordered">
      <thead class="thead-dark">
        <tr>
          <th><?= TABLE_HEADING_PRODUCTS ?></th>
          <th class="text-center"><?= TABLE_HEADING_MODEL ?></th>
          <th class="text-center"><?= TABLE_HEADING_QUANTITY ?></th>
          <th class="text-right"><?= TABLE_HEADING_PRICE ?>&nbsp;</th>
        </tr>
      </thead>
      <tbody>
      <?php
        $products_query = $db->query(sprintf(<<<'EOSQL'
SELECT p.products_id, pd.products_name, p.products_model, p.products_quantity, p.products_price, l.name
  FROM products p, products_description pd, languages l
  WHERE p.products_id = pd.products_id
    AND p.products_id = pd.products_id
    AND l.languages_id = pd.language_id
    AND pd.language_id = %s
 ORDER BY pd.products_name ASC
EOSQL
        , (int)$languages_id));

        while ( $products = $products_query->fetch_assoc() ) {
          $products_id = $products['products_id'];

          // check for product or attributes below reorder level
          $products_stock_query = $db->query(sprintf(<<<'EOSQL'
SELECT products_stock_attributes, products_stock_quantity
  FROM products_stock
  WHERE products_id = %s
  ORDER BY products_stock_attributes
EOSQL
        , (int)$products['products_id']));

          $products_stock_rows = mysqli_num_rows($products_stock_query);
          // Highlight products with low stock
          if ( $products['products_quantity'] > STOCK_REORDER_LEVEL ){
            $trclass = '';
					} else {
					  $trclass = 'table-warning';
					}

          $products_quantity= $products['products_quantity'];
          $products_price=($products_stock_rows > 0) ? '&nbsp;' : $currencies->format($products['products_price']);
          //  Add Attributes

          if ($products_stock_rows > 0) {
            $products_options_name_query = $db->query(sprintf(<<<'EOSQL'
SELECT distinct popt.products_options_id, popt.products_options_name
  FROM products_options popt, products_attributes patrib
  WHERE patrib.products_id = %s
    AND patrib.options_id = popt.products_options_id
    AND popt.products_options_track_stock = '1'
    AND popt.language_id = %s
  ORDER BY popt.products_options_id
EOSQL
        , (int)$products['products_id'], (int)$languages_id));
?>
        <tr>
          <td><?= '<a href="' . $Admin->link("catalog.php", 'pID=' . $products['products_id'] . '&action=new_product&stock_updated=true') . '">' . $products['products_name'] .'</a>' ?></td>
          <td class="text-center"><?= $products['products_model'] ?></td>
          <td class="text-center"><?= $products_quantity ?></td>
          <td class="text-right"><?= $products_price ?>&nbsp;</td>
        </tr>

        <tr>
          <td>
            <table class="table table-sm table-bordered table-light table-not-striped">
              <tr>
<?php
                // build headng line with option names
                $products_options_count = 0;
                while ( $products_options_name = $products_options_name_query->fetch_assoc() ) {
                  echo '                    <td class="text-success">' . $products_options_name['products_options_name'] . '</td>' . PHP_EOL;
                  $products_options_count++;
                }
?>
              </tr>
<?php
              // buld array of attributes price delta
              $attributes_price = array();
              $products_attributes_query = $db->query(sprintf(<<<'EOSQL'
SELECT pa.options_id, pa.options_values_id, pa.options_values_price, pa.price_prefix
  FROM products_attributes pa
  WHERE pa.products_id = %s
EOSQL
        , (int)$products['products_id']));

            while ( $products_attributes_values = $products_attributes_query->fetch_assoc() ) {
              $option_price = $products_attributes_values['options_values_price'];
              if ($products_attributes_values['price_prefix'] == "-") $option_price= -1*$option_price;
              $attributes_price[$products_attributes_values['options_id']][$products_attributes_values['options_values_id']] = $option_price;
            }

            // now display the attribute value names, table the html for quantity & price to get everything
            // to line up right
            $model_html_table = '<table class="table table-sm table-bordered table-light table-not-striped">' . PHP_EOL;
            $model_html_table .= '  <tr><td class=" colspan="' . $products_options_count . '">&nbsp;</td></tr>' . PHP_EOL;
            $quantity_html_table = '<table class="table table-sm table-bordered table-light table-not-striped">' . PHP_EOL;
            $quantity_html_table .= ' <tr><td class=" colspan="' . $products_options_count . '">&nbsp;</td></tr>' . PHP_EOL;
            $price_html_table = '<table class="table table-sm table-bordered table-light table-not-striped">' . PHP_EOL;
            $price_html_table .= '  <tr><td class=" colspan="' . $products_options_count . '">&nbsp;</td></tr>' . PHP_EOL;
            while ($products_stock_values = $products_stock_query->fetch_assoc()) {
              $attributes=explode(",", $products_stock_values['products_stock_attributes']);
              echo                   '<tr>' . PHP_EOL;

              $model_html_table .= '<tr>' . PHP_EOL;
              $quantity_html_table .= '<tr>' . PHP_EOL;
              $price_html_table .= '<tr>' . PHP_EOL;
              $total_price=$products['products_price'];

              // Highlight products out of stock
              if (($products_stock_values['products_stock_quantity']) > STOCK_REORDER_LEVEL){
                $trclassstock="";
              } else {
                $trclassstock="table-warning";
              }

              foreach ($attributes as $attribute) {
                $attr = explode("-",$attribute);
                $values = $db->query("SELECT products_options_values_name FROM products_options_values WHERE products_options_values_id = " . (int)$attr[1] . " AND language_id = " . (int)$_SESSION['languages_id']);
                $values_values = $values->fetch_assoc();

                echo '<td class="' . $trclassstock . '" >&nbsp;&nbsp;' . $values_values['products_options_values_name'] . '</td>' . PHP_EOL;
                $total_price += $attributes_price[$attr[0]][$attr[1]]?? 0;
              }
              $total_price=$currencies->format($total_price);
              echo '                  </tr>' . PHP_EOL;

              $model_html_table .= '<td class=" ' . $trclassstock . '">&nbsp;</td>' . PHP_EOL;
              $model_html_table .= '</tr>' . PHP_EOL;
              $quantity_html_table .= '<td class="text-center ' . $trclassstock . '">' . $products_stock_values['products_stock_quantity'] . '</td>' . PHP_EOL;
              $quantity_html_table .= '</tr>' . PHP_EOL;
              $price_html_table .= '<td class="text-right ' . $trclassstock . '">' . $total_price . '&nbsp;</td>' . PHP_EOL;
              $price_html_table .= '</tr>' . PHP_EOL;
            }
            echo ' </table>' . PHP_EOL;
            echo '</td>' . PHP_EOL;
            $model_html_table .= '</table>' . PHP_EOL;
            $quantity_html_table .= '</table>' . PHP_EOL;
            $price_html_table.= '</table>' . PHP_EOL;
            echo '              <td>' . $model_html_table . '</td>' . PHP_EOL;
            echo '              <td>' . $quantity_html_table . '</td>' . PHP_EOL;
            echo '              <td>' . $price_html_table . '</td>' . PHP_EOL;
            echo '            </tr>' . PHP_EOL;
          } else {
?>
            <tr class="<?php echo $trclass; ?>">
              <td><?= '<a href="' . $Admin->link("catalog.php", 'pID=' . $products['products_id'] . '&action=new_product') . '">' . $products['products_name'] .'</a>' ?></td>
              <td class="text-center"><?= $products['products_model'] ?></td>
              <td class="text-center"><?= $products_quantity ?></td>
              <td class="text-right"><?= $products_price ?>&nbsp;</td>
            </tr>
<?php
          }
////////////////////////// End Attributes
        }

?>
        <tr>
      </tbody>
    </table>
  </div>
  </div>

<?php
  require('includes/template_bottom.php');
  require('includes/application_bottom.php');
?>
