<?php
/*
  $Id$

  QTpro
  Version 7.4.6 Phoenix

  by @raiwa
  info@oscaddons.com
  www.oscaddons.com

  Copyright (c) 2021 Rainer Schmied

  CE Phoenix, E-Commerce made Easy
  https://phoenixcart.org

  Copyright (c) 2021 Phoenix Cart

  Released under the GNU General Public License
*/

  class pi_qtpro_table extends abstract_module {

    const CONFIG_KEY_BASE = 'PI_QTPRO_TABLE_';

    public $group = 'pi_modules_c';
    public $content_width;

    function __construct() {
      parent::__construct();

      $this->group = basename(dirname(__FILE__));

      if (!defined('MODULE_CONTENT_PI_QTPRO_OPTIONS_STATUS') && !defined('PI_QTPRO_OPTIONS_STATUS')) {
        $this->description .=   '<div class="secWarning">' . PI_QTPRO_TABLE_WARNING . '<br>
                                <a href="modules_content.php?module=cm_pi_qtpro_options&action=install">' . PI_QTPRO_TABLE_INSTALL_NOW . '</a></div>';
      }

      if (!file_exists(DIR_FS_CATALOG . 'templates/' . TEMPLATE_SELECTION . '/includes/hooks/shop/siteWide/qtPro.php')) {
        $this->description = '<div class="secWarning">' . PI_QTPRO_TABLE_ERROR_HOOK . '</div>' . $this->description;
        $this->enabled = false;
      }

      $this->description .= '<div class="secWarning">' . MODULE_CONTENT_BOOTSTRAP_ROW_DESCRIPTION . '</div>';

      if ( $this->enabled ) {
        $this->group = 'pi_modules_' . strtolower(PI_QTPRO_TABLE_GROUP);
        $this->content_width = (int)PI_QTPRO_TABLE_CONTENT_WIDTH;
      }

    }

    function getOutput() {
      global $product, $currencies;

      $content_width = (int)PI_QTPRO_TABLE_CONTENT_WIDTH;

      // get the attributes for this product from the product class
      $attributes = $product->get('attributes');
      // remove unstocked attributes
      foreach ($attributes as $key => $value ) {      
        if ($value['track_stock'] == 0) {
          unset($attributes[$key]);
        }
      }

      if ( [] !== $attributes ) {
        $products_attributes_stock = $GLOBALS['pad']->products_attrib_stock;
        if ( !empty($products_attributes_stock) ) {
          // build an array with the option id's as keys to use for sorting
          $attributes_keys = array_flip(array_keys($attributes));
          $product_base_price = $product->get('base_price');

          $table_body = [];
          foreach ($products_attributes_stock as $products_attribute_combs => $products_attribute_combs_stock ) {
            if ($products_attribute_combs_stock > 0) {
              //We only want to display rows for combinations we have on stock...
              //For example the quantity can be 0 or even negative if oversold.
              $comb_attributes = explode(',', $products_attribute_combs);
              // assign the option id's as array keys
              $comb_attributes_new = [];
              foreach ($comb_attributes as $key => $comb_attribute) {
                $comb_attr = explode('-', $comb_attribute);
                $comb_attributes_new[$comb_attr[0]] = $comb_attribute;
              }

              // sort the attributes array using the sort order of the attributes got from the attributes class
              $sorted_attributes = array_replace($attributes_keys, $comb_attributes_new);

              $row_names = [];
              $total_price = $product_base_price;
              foreach ($sorted_attributes as $comb_attribute) {
                $comb_attr = explode('-', $comb_attribute);

                $row_names[] = $attributes[$comb_attr[0]]['values'][$comb_attr[1]]['name'];

                if ( $attributes[$comb_attr[0]]['values'][$comb_attr[1]]['price'] > 0 ) {
                  if ( $attributes[$comb_attr[0]]['values'][$comb_attr[1]]['prefix'] == '+' ) {
                    $total_price  +=  $attributes[$comb_attr[0]]['values'][$comb_attr[1]]['price'];
                  } else {
                    $total_price  -=  $attributes[$comb_attr[0]]['values'][$comb_attr[1]]['price'];
                  }
                }
              }

              $total_price = $GLOBALS['currencies']->display_price($total_price, $product->get('tax_rate'));

              $table_body[] = [
                              'names' => $row_names,
                              'price' => $total_price,
                              'stock' => $products_attribute_combs_stock
                              ];

            }
          }

          $tpl_data = ['group' => $this->group, 'file' => __FILE__];
          include 'includes/modules/block_template.php';
        }
      }
    }

    protected function get_parameters() {
      return [
        'PI_QTPRO_TABLE_STATUS' => [
          'title' => 'Enable QT Pro Stock Table Module',
          'value' => 'True',
          'desc' => 'Should this module be shown on the product info page?',
          'set_func' => "Config::select_one(['True', 'False'], ",
        ],
        'PI_QTPRO_TABLE_GROUP' => [
          'title' => 'Module Display',
          'value' => 'C',
          'desc' => 'Where should this module display on the product info page?',
          'set_func' => "Config::select_one(['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I'], ",
        ],
        'PI_QTPRO_TABLE_CONTENT_WIDTH' => [
          'title' => 'Content Width',
          'value' => '12',
          'desc' => 'What width container should the content be shown in?',
          'set_func' => "Config::select_one(['12', '11', '10', '9', '8', '7', '6', '5', '4', '3', '2', '1'], ",
        ],
        'PI_QTPRO_TABLE_SORT_ORDER' => [
          'title' => 'Sort Order',
          'value' => '80',
          'desc' => 'Sort order of display. Lowest is displayed first.',
        ],
      ];
    }

  }
