<?php
/*
  QTPro 7.4.12 Phoenix
  by @raiwa
  raiwa@phoenixcartaddons.com
  www.phoenixcartaddons.com

      pad_sequenced_dropdowns.php

      Contribution extension to:
        osCommerce, Open Source E-Commerce Solutions
        http://www.oscommerce.com

      Copyright (c) 2021 Rainer Schmied
      Based on prior works released under the GNU General Public License:

        Copyright (c) 2004, 2005 Ralph Day

        QT Pro prior versions
          Ralph Day, October 2004
          Tom Wojcik aka TomThumb 2004/07/03 based on work by Michael Coffman aka coffman
          FREEZEHELL - 08/11/2003 freezehell@hotmail.com Copyright (c) 2003 IBWO
          Joseph Shain, January 2003
        osCommerce MS2
          Copyright (c) 2003 osCommerce

      Modifications made:
          11/2004 - Created
          12/2004 - Fix _draw_dropdown_sequence_js to prevent js error when all attribute combinations
                    are out of stock
          03/2005 - Remove '&' for pass by reference from parameters to call of
                    _build_attributes_combinations.  Only needed on method definition and causes
                    error messages on some php versions/configurations

*******************************************************************************************

      QT Pro Product Attributes Display Plugin
      pad_sequenced_dropdowns.php - Display stocked product attributes first as one dropdown for each attribute
                                    with Javascript to force user to select attributes in sequence so only
                                    in-stock combinations are seen.
      Class Name: pad_sequenced_dropdowns
      This class generates the HTML to display product attributes.  First, product attributes that
      stock is tracked for are displayed, each attribute in its own dropdown list with Javascript to
      force user to select attributes in sequence so only in-stock combinations are seen.  Then
      attributes that stock is not tracked for are displayed, each attribute in its own dropdown list.
      Methods overidden or added:
        _draw_stocked_attributes            draw attributes that stock is tracked for
        _draw_dropdown_sequence_js          draw Javascript to force the attributes to be selected in
                                            sequence
        _SetConfigurationProperties         set local properties
*/

  class pad_sequenced_dropdowns extends pad_multiple_dropdowns {

/*
    Method: _draw_stocked_attributes
    draw dropdown lists for attributes that stock is tracked for
    Parameters:
      none
    Returns:
      string:         HTML to display dropdown lists for attributes that stock is tracked for
*/
    function _draw_stocked_attributes() {

      $out = '';

      $attributes = $this->_build_attributes_array(true, false);
      if (sizeof($attributes) <= 1) {
        return parent::_draw_stocked_attributes();
      }

      // Check stock
      $s = sizeof($attributes[0]['ovals']);
      for ($a = 0; $a < $s; $a++) {
        $stock_sum = 0;
        $attributes_combination = $attributes[0]['oid'] . '-' . $attributes[0]['ovals'][$a]['id'];
        // loop through products_attrib_stock array and check for stock
        foreach ($this->products_attrib_stock as $attrib_combs => $stock) {
          if ( strpos($attrib_combs, $attributes_combination) !== false) {
            $stock_sum += $stock;
          }
        }
        if ($stock_sum < 1) {
          unset($attributes[0]['ovals'][$a]);
        }
      }

      for($o = 0; $o < sizeof($attributes); $o++) {
        $s = sizeof($attributes[$o]['ovals']);
        for ($a = 1; $a < $s; $a++) {
          $stock_sum = 0;
          $attributes_combination = $attributes[$o]['oid'] . '-' . $attributes[$o]['ovals'][$a]['id'];
          // loop through products_attrib_stock array and check for stock
          foreach ($this->products_attrib_stock as $attrib_combs => $stock) {
            if ( strpos($attrib_combs, $attributes_combination) !== false) {
              $stock_sum += $stock;
            }
          }
          $out_of_stock = ($stock_sum > 0)? false : true;
        }
      }

      // Draw first option dropdown with all values
      $out .= '<div class="form-group row">' . PHP_EOL;
      $out .= ' <label for="id[' . $attributes[0]['oid'] . ']" class="col-form-label col-sm-3 text-left text-sm-right">' . $attributes[0]['oname'] . ': </label>' . PHP_EOL;
      $out .= ' <div class="col-sm-9">' . PHP_EOL;
      $out .= (new Select('id[' . $attributes[0]['oid'] . ']', array_merge([['id'=>'', 'text'=>sprintf(MODULE_CONTENT_PI_QTPRO_OPTIONS_FIRST_SELECT, $attributes[0]['oname'])]], $attributes[0]['ovals']), ['onchange' => 'i' . $attributes[0]['oid'] . '(this.form);']))->set_selection($attributes[0]['default'])->require();
      $out .= ' </div>' . PHP_EOL;
      $out .= '</div>' . PHP_EOL;

      // Draw second to next to last option dropdowns - no values, with onchange
      for($o = 1; $o<sizeof($attributes)-1; $o++) {
        $out .= '<div class="form-group row">' . PHP_EOL;
        $out .= ' <label for="id[' . $attributes[$o]['oid'] . ']" class="col-form-label col-sm-3 text-left text-sm-right">' . $attributes[$o]['oname'] . ': </label>' . PHP_EOL;
        $out .= ' <div class="col-sm-9">' . PHP_EOL;
        $out .= (new Select('id[' . $attributes[$o]['oid'] . ']', [['id'=>'', 'text'=>sprintf(MODULE_CONTENT_PI_QTPRO_OPTIONS_NEXT_SELECT, $attributes[$o]['oname'])]], ['onchange' => 'i' . $attributes[$o]['oid'] . '(this.form);']))->require();
        $out .= ' </div>' . PHP_EOL;
        $out .= '</div>' . PHP_EOL;
      }

      // Draw last option dropdown - no values, no onchange
      $out .= '<div class="form-group row">' . PHP_EOL;
      $out .= ' <label for="id[' . $attributes[$o]['oid'] . ']" class="col-form-label col-sm-3 text-left text-sm-right">' . $attributes[$o]['oname'] . ': </label>' . PHP_EOL;
      $out .= ' <div class="col-sm-9">' . PHP_EOL;
      $out .= (new Select('id[' . $attributes[$o]['oid'] . ']', [['id'=>'', 'text'=>sprintf(MODULE_CONTENT_PI_QTPRO_OPTIONS_NEXT_SELECT, $attributes[$o]['oname'])]]))->require();
      $out .= ' </div>' . PHP_EOL;
      $out .= '</div>' . PHP_EOL;

      $this->_draw_dropdown_sequence_js($attributes);

      return $out;
    }


/*
    Method: _draw_dropdown_sequence_js
    draw Javascript to display out of stock message for out of stock attribute combinations
    Parameters:
      $attributes     array   Array of attributes for the product.  Format is as returned by
                              _build_attributes_array.
    Returns:
      string:         Javascript to force user to select stocked dropdowns in sequence
*/
    function _draw_dropdown_sequence_js($attributes) {

      $combinations = [];
      $selected_combination = 0;
      $this->_build_attributes_combinations($attributes, false, 'None', $combinations, $selected_combination);
      $draw_js_stock_array = $this->_draw_js_stock_array($combinations);
      $output = <<<EOD
<script>
var stk={$draw_js_stock_array};
EOD;
      for ($curattr = 1; $curattr<sizeof($attributes); $curattr++) {
        $attr = $attributes[$curattr];
        $attr_oid = $attr['oid'];
        $output .= <<<EOD
var txt{$attr_oid}={
EOD;
        foreach ($attr['ovals'] as $oval) {
          $oval_id = $oval['id'];
          $oval_text = $oval['text'];
          $output .= <<<EOD
{$oval_id}:'{$oval_text}',
EOD;
        }
        $output = substr($output,0,strlen($output)-1) . "};";
      }

      // js functions to set next dropdown options when a dropdown selection is made
      // do all but last attribute (nothing needs to happen when it changes)
      for ($curattr = 0; $curattr < sizeof($attributes)-1; $curattr++) {
        $attr = $attributes[$curattr];
        $attr_oid = $attr['oid'];
          $output .= <<<EOD
  function i{$attr_oid}(frm) {
EOD;
        $i = key($attributes);
        for ($i = $curattr+1; $i < sizeof($attributes); $i++) {
          $attributes_i_oid = $attributes[$i]['oid'];
          $output .= <<<EOD
    frm['id[{$attributes_i_oid}]'].length=1;
EOD;
        }
        $output .= <<<EOD
    for (opt in stk
EOD;
        for ($i = 0; $i <= $curattr; $i++) {
          $attributes_i_oid = $attributes[$i]['oid'];
          $attributes_curattr_plus_one_oid = $attributes[$curattr+1]['oid'];

          $output .= <<<EOD
[frm['id[{$attributes_i_oid}]'].value]
EOD;
        }

        $output .= <<<EOD
) {
      var o1 = new Option(txt{$attributes_curattr_plus_one_oid}[opt],opt);
      frm['id[{$attributes_curattr_plus_one_oid}]'].options[frm['id[{$attributes_curattr_plus_one_oid}]'].length] = o1;
    }
  }
EOD;
      }

      // js to initialize dropdowns to defaults if product id contains attributes (i.e. clicked through to product page from cart)
        $attributes_0_oid = $attributes[0]['oid'];
      $output .= <<<EOD
  i{$attributes_0_oid}(document.cart_quantity);
EOD;
      for($o = 1; $o<sizeof($attributes)-1; $o++) {
        $attributes_o_oid = $attributes[$o]['oid'];
        $attributes_o_default = $attributes[$o]['default'];

        if ($attributes[$o]['default'] != '') {
          $output .= <<<EOD
  document.cart_quantity['id[{$attributes_o_oid}]'].value={$attributes_o_default};
  i{$attributes_o_oid}(document.cart_quantity);
EOD;
        }
        else break;
      }
      if (($o == sizeof($attributes)-1) && ($attributes[$o]['default'] != '')) {
        $attributes_o_oid = $attributes[$o]['oid'];
        $attributes_o_default = $attributes[$o]['default'];
        $output .= <<<EOD
  document.cart_quantity['id[{$attributes_o_oid}]'].value={$attributes_o_default};
EOD;
      }

      // js to not allow add to cart if selections not made
      $output .= <<<EOD
  function chksel() {
    var ok=true;
EOD;
      foreach ($attributes as $attr) {
        $attr_oid = $attr['oid'];
        $show_please_select = $this->show_please_select;
        $output .= <<<EOD
    if (this['id[{$attr_oid}]'].value==0) ok=false;
EOD;
      }
      $output .= <<<EOD
    if (!ok) alert('{$show_please_select}');
    return ok;
  }
  document.cart_quantity.onsubmit=chksel;
</script>
EOD;

      $GLOBALS['Template']->add_block($output, 'footer_scripts');
    }


/*
    Method: _SetConfigurationProperties
    Set local configuration properties
    Parameters:
      $prefix      sting     Prefix for the osCommerce DB constants
    Returns:
      nothing
*/
    function _SetConfigurationProperties($prefix) {

      // These properties are not used directly by this class
      // They are set to match how this class displays for the case of a single
      // attribute where the parent class _draw_stocked_attributes method is called
      $this->show_out_of_stock    = 'False';
      $this->mark_out_of_stock    = 'Right';
      $this->out_of_stock_msgline = 'False';
      $this->no_add_out_of_stock  = 'True';
      $this->combine_non_stock  = constant($prefix . 'COMBINE_NON_STOCKED');
      $this->options_plugin  = constant($prefix . 'PLUGIN');
      $this->actual_price_pull_down  = constant($prefix . 'ACTUAL_PRICE_PULL_DOWN');
      $this->show_please_select  = constant($prefix . 'SHOW_PLEASE_SELECT');
    }
  }
