<?php
/*
  QTPro 7.4.12 Phoenix
  by @raiwa
  raiwa@phoenixcartaddons.com
  www.phoenixcartaddons.com

      pad_single_dropdown.php

      Contribution extension to:
        osCommerce, Open Source E-Commerce Solutions
        http://www.oscommerce.com

      Copyright (c) 2021 Rainer Schmied
      Based on prior works released under the GNU General Public License:

        Copyright (c) 2004, 2005 Ralph Day

        QT Pro & CPIL prior versions
          Ralph Day, October 2004
          Tom Wojcik aka TomThumb 2004/07/03 based on work by Michael Coffman aka coffman
          FREEZEHELL - 08/11/2003 freezehell@hotmail.com Copyright (c) 2003 IBWO
          Joseph Shain, January 2003
          osCommerce MS2
          Copyright (c) 2003 osCommerce

      Modifications made:
          11/2004 - Created
          12/2004 - Fix _draw_out_of_stock_message_js to add semicolon to end of js stock array
          03/2005 - Remove '&' for pass by reference from parameters to call of
                    _build_attributes_combinations.  Only needed on method definition and causes
                    error messages on some php versions/configurations

*******************************************************************************************

      QT Pro Product Attributes Display Plugin
      pad_single_dropdown.php - Display stocked product attributes as a single dropdown with entries
                                for each possible combination of attributes.
      Class Name: pad_single_dropdown
      This class generates the HTML to display product attributes.  First, product attributes that
      stock is tracked for are displayed in a single dropdown list with entries for each possible
      combination of attributes..  Then attributes that stock is not tracked for are displayed,
      each attribute in its own dropdown list.
      Methods overidden or added:
        _draw_stocked_attributes            draw attributes that stock is tracked for
        _draw_out_of_stock_message_js       draw Javascript to display out of stock message for out of
                                            stock attribute combinations

*/

  class pad_single_dropdown extends pad_base {


/*
    Method: _draw_stocked_attributes
    draw dropdown lists for attributes that stock is tracked for
    Parameters:
      none
    Returns:
      string:         HTML to display dropdown lists for attributes that stock is tracked for
*/
    function _draw_stocked_attributes() {

      $out = '';
      $combine_nostock = ($this->combine_non_stock == 'True')? true : false;
      $attributes = $this->_build_attributes_array(true, $combine_nostock);
      if (sizeof($attributes) > 0) {
        $combinations = [];
        if ( $this->show_please_select == 'True' ) {
          // BEGIN product optionsimages support
          if ($this->options_images) {
            $combinations[] =  ['id'=> '', 'text'=> PULL_DOWN_DEFAULT, 'image' => ''];
          } else {
            $combinations[] =  ['id'=> '', 'text'=> PULL_DOWN_DEFAULT];
          }
          // END product optionsimages support
        }
        $selected_combination = 0;
        $this->_build_attributes_combinations($attributes, $this->show_out_of_stock == 'True', $this->mark_out_of_stock, $combinations, $selected_combination);

        $combname = '';
        foreach ($attributes as $attrib) {
          $combname .= ', ' . $attrib['oname'];
        }
        $combname=substr($combname,2);
        $out .= '<div class="form-group col">' . PHP_EOL;
        $out .= ' <label for="attrcomb" class="col-form-label">' . $combname . ': </label>' . PHP_EOL;
        // BEGIN product optionsimages support
        if ($this->options_images) {
          $out .= $this->qtpro_draw_pull_down_menu_single_options('attrcomb', $combinations, $combinations[$selected_combination]['id'], ' id=option' . $combinations[$selected_combination]['id'] . ' required aria-required="true"') . '<br>' . "\n";
        } else {
          $out .= (new Select('attrcomb', $combinations, ['id' => 'option' . $combinations[$selected_combination]['id']]))->set_selection($combinations[$selected_combination]['id'])->require();
        }
        // END product optionsimages support
        $out .= '</div>' . PHP_EOL;
      }

      if($this->out_of_stock_msgline == 'True')
        $this->_draw_out_of_stock_message_js($attributes);

      return $out;
    }


/*
    Method: _draw_out_of_stock_message_js
    draw Javascript to display out of stock popup message if an attempt is made to add an out of
    stock attribute combination to the cart
    Parameters:
      $attributes     array   Array of attributes for the product.  Format is as returned by
                              _build_attributes_array.
    Returns:
      string:         Javascript to display out of stock message for out of stock attribute combinations
*/
    function _draw_out_of_stock_message_js($attributes) {

      if (($this->show_out_of_stock == 'True') && ($this->no_add_out_of_stock == 'True')) {
        $output = null;
        $combinations = [];
        $selected_combination = 0;
        $this->_build_attributes_combinations($attributes, false, 'None', $combinations, $selected_combination);
        $draw_array_combinations = $this->_draw_js_stock_array($combinations);
        $sizeof_attributes = sizeof($attributes);
        $out_of_stock_message = MODULE_CONTENT_PI_QTPRO_OPTIONS_OUT_OF_STOCK_MESSAGE;

        $output .= <<<EOD
<script>
  function chkstk(frm) {
    var stk={$draw_array_combinations};
    var instk=false;
    if (frm.attrcomb.type=='select-one') {
      var attrs=frm.attrcomb.value.split(',');
    }
    else {
      for (i=0; i,frm.attrcomb.length; i++) {
        if (frm.attrcomb[i].checked) {
          var attrs=frm.attrcomb[i].value.split(',');
          break;
        }
      }
    }
    var id=Array({$sizeof_attributes});
    for (i=0; i<attrs.length; i++) {
      id[i]=attrs[i].split('-')[1];
    }
EOD;
        for ($i = 0; $i < sizeof($attributes); $i++) {
          $output .= <<<EOD
  if (stk
EOD;
          for ($j = 0; $j <= $i; $j++) {
            $output .= <<<EOD
[id[{$j}]]
EOD;
          }
          $output .= <<<EOD
)
EOD;
        }
          $output .= <<<EOD
instk=true;
    return instk;
  }
EOD;
        if ($this->no_add_out_of_stock == 'True') {
          $output .= <<<EOD
  function chksel() {
    var instk=chkstk(document.cart_quantity);
    if (!instk) alert('{$out_of_stock_message}');
    return instk;
  }
  document.cart_quantity.onsubmit=chksel;
EOD;
        }
          $output .= <<<EOD
</script>
EOD;

        $GLOBALS['Template']->add_block($output, 'footer_scripts');
      }
    }

////
// Output a form pull down menu for Option Images BS
    function qtpro_draw_pull_down_menu_single_options($name, $values, $default = '', $parameters = '', $required = false) {
      if ($this->options_images) {
        $field = '<select name="' . htmlspecialchars($name) . '"';
        if (!Text::is_empty($parameters)) $field .= ' ' . $parameters;
        $field .= ' class="form-control selectpicker">';
        if (empty($default) && ( (isset($_GET[$name]) && is_string($_GET[$name])) || (isset($_POST[$name]) && is_string($_POST[$name])) ) ) {
          if (isset($_GET[$name]) && is_string($_GET[$name])) {
            $default = stripslashes($_GET[$name]);
          } elseif (isset($_POST[$name]) && is_string($_POST[$name])) {
            $default = stripslashes($_POST[$name]);
          }
        }

        for ($i=0, $n=sizeof($values); $i<$n; $i++) {
          $field .= '<option value="' . htmlspecialchars($values[$i]['id']) . '"';
          if ($default == $values[$i]['id']) {
            $field .= ' selected="selected"';
          }
          $option_name = htmlspecialchars(str_replace(['"', '\'', '<', '>'], ['&quot;', '&#039;', '&lt;', '&gt;'], $values[$i]['text']));

          if( !empty($values[$i]['image']) ) {
            $field .= ' data-content=\'';
            for ($j=0, $k=sizeof($values[$i]['image']); $j<$k; $j++) {
              if ( !Text::is_empty($values[$i]['image'][$j]) ) {
                $field .= new Image('images/options/' . $values[$i]['image'][$j], ['title' => $option_name, 'width' => '40', 'height' => '30']) . '&nbsp;';
              }
            }
            $field .= $option_name . '\'';
          }
          $field .= '>' . $option_name . '</option>';
        }
        $field .= '</select>';

        if ($required == true) $field .= FORM_REQUIRED_INPUT;

        return $field;
      }
    }
  }
