<?php
/*
  QTPro 7.4.0 Phoenix

      pad_single_radioset.php

      Contribution extension to:
        osCommerce, Open Source E-Commerce Solutions
        http://www.oscommerce.com

      Copyright (c) 2021 Rainer Schmied
      Based on prior works released under the GNU General Public License:

        Copyright (c) 2004, 2005 Ralph Day

        QT Pro & CPIL prior versions
          Ralph Day, October 2004
          Tom Wojcik aka TomThumb 2004/07/03 based on work by Michael Coffman aka coffman
          FREEZEHELL - 08/11/2003 freezehell@hotmail.com Copyright (c) 2003 IBWO
          Joseph Shain, January 2003
          osCommerce MS2
          Copyright (c) 2003 osCommerce

          Modifications made:
          11/2004 - Created
          03/2005 - Remove '&' for pass by reference from parameters to call of
                    _build_attributes_combinations.  Only needed on method definition and causes
                    error messages on some php versions/configurations

*******************************************************************************************

      QT Pro Product Attributes Display Plugin
      pad_single_radioset.php - Display stocked product attributes as a single radioset with entries
                                for each possible combination of attributes.
      Class Name: pad_single_radioset
      This class generates the HTML to display product attributes.  First, product attributes that
      stock is tracked for are displayed in a single radioset with entries for each possible
      combination of attributes..  Then attributes that stock is not tracked for are displayed,
      each attribute in its own dropdown list.
      Methods overidden or added:
        _draw_stocked_attributes             draw attributes that stock is tracked for
*/

  if (!class_exists('pad_single_radioset')) {
    class pad_single_radioset extends pad_single_dropdown {


  /*
      Method: _draw_stocked_attributes
      draw dropdown lists for attributes that stock is tracked for
      Parameters:
        none
      Returns:
        string:         HTML to display dropdown lists for attributes that stock is tracked for
  */
      function _draw_stocked_attributes() {

        $out= '' ;
        $combine_nostock = ($this->combine_non_stock == 'True')? true : false;
        $attributes = $this->_build_attributes_array(true, $combine_nostock);
        if (sizeof($attributes) > 0) {
          $combinations = [];
          $selected_combination = 0;
          $this->_build_attributes_combinations($attributes, $this->show_out_of_stock == 'True', $this->mark_out_of_stock, $combinations, $selected_combination);

          $combname = '';
          foreach ($attributes as $attrib) {
            $combname .= ', ' . $attrib['oname'];
          }
          $combname = substr($combname,2).': ';

          $out .= '<div class="table">' . PHP_EOL;
          $out .= ' <table class="table table-striped table-condensed table-hover">';
          $out .= '   <thead>' . PHP_EOL;
          $out .= '     <tr>' . PHP_EOL;
          $out .= '       <th>' . $combname . '</th>' . PHP_EOL;
          $out .= '     </tr>' . PHP_EOL;
          $out .= '   <thead>' . PHP_EOL;
          $out .= ' <tbody>' . PHP_EOL;

          foreach ($combinations as $combindex => $comb) {
            // BEGIN product optionsimages support
            if ($this->options_images) {
              if ( [] !== $comb['image'] ) {
                $images = null;
                for ($j=0, $k=sizeof($comb['image']); $j<$k; $j++) {
                  if ( !Text::is_empty($comb['image'][$j]) ) {
                    $images .= new Image('images/options/' . $comb['image'][$j], ['title' => $comb['text'], 'width' => '40', 'height' => '30']) . ' ';
                  }
                }
              }
              $comb['text'] = $images . $comb['text'];
            }
            // END product optionsimages support
            $out .= '   <tr class="table-selection">' . PHP_EOL;
            $out .= '     <td>' . (new Tickable('attrcomb', ['value' => $combinations[$combindex]['id']], 'radio'))->tick($combindex == $selected_combination) . ' ' . $comb['text'] . '</td>' . PHP_EOL;
            $out .= '   </tr>' . PHP_EOL;
          }
          $out .= '   </tbody>' . PHP_EOL;
          $out .= ' </table>' . PHP_EOL;
          $out .= '</div>' . PHP_EOL;
          $combname='';
        }
        return $out;
      }

    }
  }
